function [all_finalT, all_err, all_C1, all_Na, all_numCntrlPts] = runWithRandom(numRandomTries, D, startT, ...
						  allTimes, numFinalSamples, ...
						  classes, saveFileName, sigma)
% [all_finalT, all_err, all_C1, all_Na, all_numCntrlPts] = runWithRandom(numRandomTries, D, startT, allTimes, numFinalSamples, classes, saveFileName, sigma)
% 
% given a simulated dataset, number of final samples (a1), number of
% random tries (b1), an initial set of
% points etc., this function randomly generates b1 sets of timepoints
% with a1 points each and evaluates error for those points
% input:
% numRandomTries: number of random samples to be generated
% D: numGenes x numTotalSampled : the entire dataset
% startT: 1 x numInitiallySampled:  the index of starting set of sampled time-points (numInitiallySampled >=5)
% allTimes: all possible sampling locations
% numFinalSamples: number of samples to be taken finally
% classes: number of categories
% strategy: 'gcv-activelearning' only implmented currently
% saveFileName: the file name to which other auxiliary data should be saved
% sigma: the estimated variance. optional, set to 0.2
% output:
% all_finalT: numRandomTries x numFinalSamples : the final sets of sampled time-points
% all_err: 1 x numRandomTries  : total (final) error in each of the random tries
% all_C1: numRandomTries x numGenes x numTotalSampled : estimated function values
% all_Na: numRandomTries x numGenes: the classification corresponding to all_C1
% all_numCntrlPts : 1 x numRandomTries : optimal number of control
% points used, finally

if (nargin < 8)
  sigma =  0.2;
end

if (size(startT,1)==1)
  startT = startT';
end

if (size(allTimes,1)==1)
  allTimes = allTimes';
end

[numGenes, numTotalSampled] = size(D);
if (startT(1)~=1 | startT(end) ~= numTotalSampled | size(startT,1) <5 | ...
    numTotalSampled ~= length(allTimes))
  error('runWithRandom.m: bad startT or allTimes');
end
initialN = length(startT);

if (numFinalSamples <= initialN)
  error('runWithRandom.m: need numFinalSamples > size(startT)');
end

all_finalT = zeros(numRandomTries, numFinalSamples);
all_err = zeros(1,numRandomTries); 
all_C1 = zeros(numRandomTries, numGenes, numTotalSampled);
all_Na = zeros(numRandomTries, numGenes);
all_numCntrlPts = zeros(1,numRandomTries);

idxs = setdiff(1:numTotalSampled, startT);
r = length(idxs);
R = numFinalSamples - initialN;
for k=1:numRandomTries
  x1 = randperm(r);
  x2 = idxs(x1(1:R));
  x3 = [x2'; startT];
  sT = sort(x3);
  all_finalT(k,:) = sT';
  
  [gcvI, Agcv, C1gcv, C2gcv, Na_gcv, gcvScore, sig] = findLambdaG(D, sT, ...
						  allTimes, classes);
  all_numCntrlPts(k) = gcvI;
  all_C1(k,:,:) = C1gcv;
  all_Na(k,:) = Na_gcv;
  
  currErr = 0;
  for j=1:numFinalSamples-1
    ci1 = CI( Agcv(j,j), sqrt(sig));
    ci2 = CI( Agcv(j+1,j+1), sqrt(sig));
    currErr = currErr+ (ci1+ci2) * 0.5 * (allTimes(sT(j+1)) - allTimes(sT(j)));
  end
  all_err(k) = currErr;
end


save(saveFileName, 'all_finalT', 'all_err', 'all_C1', 'all_Na', ...
     'all_numCntrlPts', 'D', 'allTimes', 'numFinalSamples', 'startT', 'classes', 'allTimes', 'numRandomTries');
