function [numFinalSamples, finalT, err, finalCI, C1, Na, numCntrlPts] = runWithThreshold(D, startT, ...
						  allTimes, errThreshold, classes, ...
						  strategy, saveFileName, ...
						  sigma)
% many mods here for alpha data, see t7_run.m
%
% [finalT, finalD, C1, Na, numCntrlPts] = runSimulation(D, startT,
% allTimes, numFinalSamples, classes, strategy, sigma, saveFileName)
% 
% given a simulated dataset and a strategy, run simulations to find the
% optimal best sampling strategy
% input:
% D: numGenes x numTotalSampled : the entire dataset
% startT: 1 x numInitiallySampled:  the index of starting set of sampled time-points (numInitiallySampled >=5)
% allTimes: all possible sampling locations
% errThreshold: the error threshold (may be specific to the method used)
% classes: number of categories
% strategy: 'gcv-activelearning' only implmented currently
% saveFileName: the file name to which other auxiliary data should be saved
% sigma: the estimated variance. optional, set to 0.2
% output:
% numFinalSamples: number of sample points chosen finally
% finalT: 1 x numFinalSamples : the final set of sampled time-points
% err: 1 x numTotalSampled : total error ( as per the strategy chosen)
% at timepoint i. err(numFinalSamples+1..end) is all -1;
% finalCI: 1 x numTotalSampled : the C.I.s for the data set
% C1: numGenes x numTotalSampled: estimated function values at each of
% the could-be-sampled time-points.
% Na: numGenes x 1: the classification corresponding to C1
% numCntrlPts : the optimal number of control points used, finally.
global topLevelCost;
global adaptiveCostStrategy;
global imposeGCVcpLimit;
global DEBUG;
global lcvalcluster_threshold;
global lcvalcluster_minCPs;

if (nargin < 8)
  sigma =  0.2;
end

if (size(startT,1)==1)
  startT = startT';
end

if (size(allTimes,1)==1)
  allTimes = allTimes';
end

[numGenes, numTotalSampled] = size(D);
if (startT(1)~=1 | startT(end) ~= numTotalSampled | size(startT,1) <5 | ...
    numTotalSampled ~= length(allTimes))
  error('runWithThreshold.m: bad startT or allTimes');
end

lcvalcluster_threshold = errThreshold;

strategy
realTopLevelCost = topLevelCost;

initialN = size(startT,1);

numCurrSamples = initialN;

% the i-th row of sampledPoints contains a bit-vector describing which
% of i time-points were sampled
sampledPoints= zeros(numTotalSampled,numTotalSampled);
sampledPoints(initialN, startT) = 1;

% gcvCntrlPts(i) indicates optimal # cntrl points when |T|=i
gcvCntrlPts = zeros(numTotalSampled,1);

% gcvC1, gcvNa indicate estimate and classification when |T|=i
gcvC1 = zeros(numTotalSampled, numGenes, numTotalSampled);
gcvNa = zeros(numTotalSampled, numGenes);

% bestC1(i), bestNa(i) indicate estimate and classification which can be
% obtained by using all the numTotalSampled samples but using
% gcvCntrlPts(i) control points
bestC1 = zeros(numTotalSampled, numGenes, numTotalSampled);
bestNa = zeros(numTotalSampled, numGenes);

% ci(i) indicates the confidence intervals at when i time-points have
% been sampled
ci = zeros(numTotalSampled,numTotalSampled);

% err(i) indicates the total error when |T|=i
err = 10000 * ones(1,numTotalSampled);

T = startT
gcvCP = 5;
lcvCP = zeros(size(T));
lcvalclusterAll = {};
done = 0;
while ((numCurrSamples < 8) | (done==0 & (numCurrSamples < numTotalSampled)))
  adaptiveCostStrategy
  
  [nextI, xnumCntrlPts, xlcvCntrlPts, xfinalCI, xC1, xNa, sig, lcvalclusterdat] = pickNext( ...
      D(:,T), T, allTimes, classes, strategy, sigma, gcvCP);
  
  if (strfind(strategy,'gcv-activelearning'))
    currS = getS(allTimes(T), xnumCntrlPts);
    currA = currS * pinv(currS);
  end
  
  if (strfind(strategy,'lcv-al-cluster'))
    currS = getS(allTimes(T), lcvalclusterdat{end-4});
    currA = currS * pinv(currS);
  end
  
  currErr = 0;
  n = max(size(T));
  for k=1:n-1
    ci1 = CI( currA(k,k), sqrt(sig));
    ci2 = CI( currA(k+1,k+1), sqrt(sig));
    currErr = currErr+ (ci1+ci2) * 0.5 * ( allTimes(T(k+1)) -  allTimes(T(k)) );
  end
  
  if (length(lcvalcluster_minCPs)>0)
    cpMult = errThreshold/ (2* (lcvalcluster_minCPs-5));
    currErr
    err(numCurrSamples) = currErr + cpMult*max([lcvalcluster_minCPs-xnumCntrlPts,0]);
  else
    err(numCurrSamples) = currErr;
  end
  totalErrThisTime = err(numCurrSamples)
  
  gcvCntrlPts(numCurrSamples) = xnumCntrlPts;

  gcvC1(numCurrSamples,:,:) =xC1;
  gcvNa(numCurrSamples,:) = xNa;

  
  %[yC1,yC2,yNa, sig2] = contRep(D, D, allTimes', allTimes', gcvCntrlPts(numCurrSamples)-2, classes);
  %bestC1(numCurrSamples,:,:) = yC2(:,allTimes-allTimes(1)+1);
  %bestNa(numCurrSamples,:) = yNa;
  
  ci(numCurrSamples,:) = xfinalCI;
  
  if (imposeGCVcpLimit)
    gcvCP = xnumCntrlPts;
  else
    gcvCP = 0;
  end
  
  if (adaptiveCostStrategy)
    if (gcvCntrlPts(numCurrSamples)==gcvCntrlPts(numCurrSamples-1))
      if (strfind(strategy,'lcv-al-cluster'))
	% changed from 0.6*realTopLevelCost for alphaData oct 27
	topLevelCost = max([ 0.2*realTopLevelCost, 0.8 * topLevelCost])
      else
	topLevelCost = max([ 0.7*realTopLevelCost, 0.9 * topLevelCost])
      end
    else
      topLevelCost = realTopLevelCost
    end
  end
  
  if (strfind(strategy,'gcv-activelearning'))
    if (err(numCurrSamples) > errThreshold)
      done=1;
    else
      done=0;
    end
  end

  errorsTillNow= err(initialN:numCurrSamples)
  ncs=numCurrSamples;
  if (strfind(strategy,'lcv-al-cluster'))
    if ( max(err(ncs-1:ncs))<errThreshold & (max(err(ncs-1:ncs))- ...
					     min(err(ncs-1:ncs)) < errThreshold/10))
      done=1;
    else
      done=0;
    end
    
    if (err(ncs) < errThreshold)
      DEBUG=3;
    else
      DEBUG=1;
    end
  end

  C1= xC1;%(:, allTimes-allTimes(1)+1);
  Na =xNa;
  numCntrlPts = xnumCntrlPts;
  finalT = T;  
  finalCI = xfinalCI;
  interrupted =1;
  sampledPoints(numCurrSamples,T) = 1;
  
  if (length(strfind(strategy,'lcv'))>0)
    lcvCntrlPts(numCntrlPts,T) = xlcvCntrlPts;
  end
  lcvalclusterAll{end+1} = lcvalclusterdat;
  
  save(saveFileName, 'interrupted', 'xnumCntrlPts', 'sampledPoints', ...
       'gcvCntrlPts','gcvNa', 'gcvC1', 'bestNa', 'bestC1', 'ci', ...
       'lcvCntrlPts', 'D', 'ncs', 'startT', 'classes', 'strategy', 'lcvalclusterAll','sig');
  
  numCurrSamples = numCurrSamples + 1;

  oldT = T;
  a = T(T<nextI);
  b = T(T>nextI);
  T = [a; nextI; b];

end
C1= xC1;
Na = xNa;
numCntrlPts = xnumCntrlPts;
finalT = oldT;  
finalCI = xfinalCI;
numFinalSamples = numCurrSamples -1;


save(saveFileName, 'finalT', 'finalCI', 'C1', 'Na', 'numCntrlPts', ...
     'sampledPoints', 'gcvCntrlPts','gcvNa', 'gcvC1', 'bestNa', 'bestC1', ...
     'ci', 'err', 'D',  'lcvCntrlPts', 'D','allTimes', 'numFinalSamples', 'startT', 'classes', 'strategy','sig','lcvalclusterAll');
